use errors::*;

extern crate serde_json as json;

use std::collections::HashMap;
use std::collections::hash_map::Iter as HashMapIter;
use std::fs::File;
use std::path::PathBuf;
use path;

const CURRENT_VERSION: u8 = 1;

#[derive(Debug, PartialEq, Eq, Serialize, Deserialize)]
pub struct FileOptions {
    pub ignore: bool,
}

#[derive(Debug, PartialEq, Eq, Serialize, Deserialize)]
pub struct Files(HashMap<PathBuf, FileOptions>);

impl Files {
    pub fn has(&self, name: &PathBuf) -> bool {
        self.0.contains_key(name)
    }

    pub fn add(&mut self, name: &PathBuf, options: FileOptions) {
        self.0.insert(name.clone(), options);
    }

    pub fn remove(&mut self, name: &PathBuf) {
        self.0.remove(name);
    }

    pub fn iter(&self) -> HashMapIter<PathBuf, FileOptions> {
        self.0.iter()
    }
}

#[derive(Debug, PartialEq, Eq, Serialize, Deserialize)]
pub struct Config {
    pub version: u8,
    pub files: Files,
}

impl Config {
    pub fn new() -> Self {
        Config {
            version: CURRENT_VERSION,
            files: Files(HashMap::new()),
        }
    }

    pub fn read() -> Result<Self> {
        Ok(json::from_reader(File::open(path::repo_file()?)?)?)
    }

    pub fn write(&self) -> Result<()> {
        Ok(json::to_writer_pretty(
            File::create(path::repo_file()?)?,
            self,
        )?)
    }
}
