use errors::*;

use std::fs;
use std::os;
use std::path::PathBuf;

#[cfg(target_os = "windows")]
pub fn symlink(force: bool, from: &PathBuf, to: &PathBuf) -> Result<()> {
    if force {
        remove_all(to)?;
    }

    ensure_tree(&to)?;

    Ok(os::windows::fs::symlink_file(from, to)?)
}

#[cfg(not(target_os = "windows"))]
pub fn symlink(force: bool, from: &PathBuf, to: &PathBuf) -> Result<()> {
    if force {
        remove_all(to)?;
    }

    ensure_tree(&to)?;

    Ok(os::unix::fs::symlink(from, to)?)
}

pub fn rename(force: bool, from: &PathBuf, to: &PathBuf) -> Result<()> {
    if force {
        remove_all(to)?;
    }

    ensure_tree(to)?;

    Ok(fs::rename(from, to)?)
}

pub fn ensure_tree(file: &PathBuf) -> Result<()> {
    match file.parent() {
        Some(v) => Ok(fs::create_dir_all(v)?),
        None => Ok(()),
    }
}

pub fn remove_all(file: &PathBuf) -> Result<()> {
    if file.exists() {
        return Ok(fs::remove_file(file)?);
    }

    Ok(())
}
