use errors::*;

use std::env;
use std::path::PathBuf;
use s_app_dir::{AppDir, XdgDir};

const APP_NAME: &'static str = "koi";

pub struct FilePath {
    pub name: PathBuf,
    pub home: PathBuf,
    pub repo: PathBuf,
}

impl FilePath {
    pub fn locate(file: &String) -> Result<Self> {
        FilePath::locate_buf(&PathBuf::from(file))
    }

    pub fn locate_buf(buf: &PathBuf) -> Result<Self> {
        if buf.starts_with(home_dir()?) {
            Ok(FilePath {
                name: PathBuf::from("~").join(&buf.strip_prefix(&home_dir()?)?),
                home: buf.clone(),
                repo: repo_dir()?.join(&buf.strip_prefix(&home_dir()?)?),
            })
        } else if buf.starts_with(repo_dir()?) {
            Ok(FilePath {
                name: PathBuf::from("~").join(&buf.strip_prefix(&repo_dir()?)?),
                home: home_dir()?.join(&buf.strip_prefix(&repo_dir()?)?),
                repo: buf.clone(),
            })
        } else if buf.starts_with("~") {
            Ok(FilePath {
                name: buf.clone(),
                home: home_dir()?.join(&buf.strip_prefix("~")?),
                repo: repo_dir()?.join(&buf.strip_prefix("~")?),
            })
        } else {
            Err(format!("{:?}: unexpected parent directory", buf).into())
        }
    }
}

pub fn home_dir() -> Result<PathBuf> {
    match env::home_dir() {
        Some(v) => Ok(v),
        None => Err("cannot find configuration directory".into()),
    }
}

pub fn repo_dir() -> Result<PathBuf> {
    let dir = match AppDir::new(APP_NAME).xdg_dir(XdgDir::Config) {
        Some(v) => v,
        None => return Err("cannot find configuration directory".into()),
    };

    Ok(dir)
}

pub fn repo_file() -> Result<PathBuf> {
    Ok(repo_dir()?.join("_repo.json"))
}
